/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of this project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <asm/mmap.h>

	.text
	.global _prolog

_prolog:

	/* jump over exceptions vectors */

	b _rs_init

	/* INTERRUPTS */

 	. = 0x100
	.global _except_vecs_start
_except_vecs_start:

	/*  0x100 sys_reset */
	li r13, 0x100
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x200
	li r13, 0x200
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x300
	li r13, 0x300
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x400
	li r13, 0x400
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x500

	/* early init */
	mtsprg1	sp
	mtsprg2	r2

	mfcr r2
	stwu r2,-4(sp)
	mflr r2
	stwu r2,-4(sp)

	lis r2, _ext_interrupt@h
	ori r2, r2, _ext_interrupt@l
	mtlr r2
	blr

	/* ========== */

	. = 0x600
	li r13, 0x600
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x700
	li r13, 0x700
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x800
	li r13, 0x800
	lis r20, except_def_hdlr@h
	ori r20, r20, except_def_hdlr@l
	mtlr r20
	blr

	. = 0x900	/* time base / decrementer MSR[EE] */
	rfi

	/* ... */

	.global _except_vecs_end
_except_vecs_end:

	. = 0x3000
_rs_init:

	/* disable exceptions & interrupts */

	/* ... */

	/* ICACHE INIT */

	/* flush and disable instruction cache if enabled */

	/* DCACHE INIT */

	/* flush and disable data cache if enabled */

	/* RELOCATE CODE & DATA */

	/* ... */

	/* binary load in ram at exec location / no need for relocation now */

	/* ZERO BSS/SBSS -- bss and sbss are assumed to be adjacent
	 * and between __bss_start and _end.
	 */

	lis r20, (__bss_start - 4)@ha
	addi r20, r20, (__bss_start - 4)@l
	lis	r21, (_end - 4)@ha
	addi r21, r21, (_end - 4)@l
	xor r0,r0,r0
	cmp r0, r20, r21
	beq _bss_skip
0:  stwu r0, 4(r20)
	cmp r0, r20, r21
	bne 0b
_bss_skip:

	/* sda sections disabled / r2 & r13 are zeroed  */

    lis  r2, (0xdeadbeef)@ha
    addi r2, r2, (0xdeadbeef)@l		/* sdata2 pointer not used */

    lis  r13, (0xdeadbeef)@ha
    addi r13, r13, (0xdeadbeef)@l

	/* JUMP TO RELOC ADDR */

	/* ... */	

	/* COPY EXCEPTION ROUTINE(S) */

	lis r20, (_except_vecs_start - 4)@ha
	addi r20, r20, (_except_vecs_start - 4)@l
	lis	r21, (_except_vecs_end - 4)@ha
	addi r21, r21, (_except_vecs_end - 4)@l
	lis	r22, (0x100 - 4)@ha
	addi r22, r22, (0x100 - 4)@l

	/* ... skip if address is already correct */
	/* r0 = old link register / not = 0 (=> MIPS cores ABI)... */
	/* li r0, 0 */

1:	lwzu r23, 4(r20)
	stwu r23, 4(r22)
	cmp  r0, r20, r21
	bne	 1b

	/* STACK INIT (r1) */

	lis	 sp, SYS_INIT_SP@ha
	addi sp, sp, SYS_INIT_SP@l

	/* linkage area */
	addi sp, sp, -8
	/* zero top two words for call back chain. ? */
	stw  r0, 0(sp)
	stw  r0, 4(sp)

	/*
	 * Call tiny -- never returns
	 */

	b  tiny

	/* NEVER RETURNS -- but branch to the _prolog just
	 * in case ;-)
	 */
	b  _prolog

	.data
	.globl	version_string

version_string:
	.ascii "TINY"
	.ascii " (", __DATE__, " - ", __TIME__, ")", "\0"

