/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of this project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <asm/mmap.h>
#include <sysutil.h>

#include <sysutil.h>
#include <s29gl.h>
#include <diu.h>
#include <timer.h>
#include <serial.h>
#include <channel.h>
#include <fbcon.h>

#include <vt100.h>
#include <vesa.h>

#include <pforth.h>
#include <pfdicdat.h>

#include <data.h>

#include <e300.h>

void uboot_restart(void)
{
   /* rewrite => use reset module instead... */
   /* 00E00 – 00EFF Reset module (RESET)  */

   /* u-boot boot_cold sym address: 0xfff003000 */

   /* uboot vars */
   #define BOOTFLAG_COLD	0x01	/* Normal Power-On: Boot from FLASH */
   #define BOOTFLAG_WARM	0x02	/* Software reboot */

   __asm__ __volatile__ ("li r21, %0\n\t" \
                         "lis r13, (0xfff00100)@ha\n\t" \
                         "addi r13, r13, (0xfff00100)@l\n\t" \
                         "mtlr r13\n\t" \
                         "blr" \
                         : /* no output */ \
                         : "i" (BOOTFLAG_COLD) );

   /* BOOTFLAG_XXXX has no effect here ... */
   /* void (*restart)(void) = 0xfff003000; also works ... */
}

void mcheck_except(void)
{
	puts("--unhandled machine check exception--\n");
/*
	printf("srr0: 0x%08x\n", e300_spr_rd(SRR0));
	printf("srr1: 0x%08x\n", e300_spr_rd(SRR1));
*/
	for(;;);
}

void except_def_hdlr(void)
{
	puts("--unhandled exception--\n");
	printf("vector: 0x%08x\n", e300_rd(r13));
	printf("msr   : 0x%08x\n", e300_msr_rd());
	puts("------sys. halted------\n");

	for(;;);
}

void tiny (void)
{
	unsigned long i;

	unsigned int* ps;
	unsigned int* pe;

	ipic_t* ipic;
	dpy_mode_t dpy;

	channel_init();

	/* configure default channels */
	channel_register(STDIN, (channel_handler_t*)uart_getc, \
					(void*)(MEM_IMMR | IO_PSC3_OFFSET));
	channel_register(STDOUT, (channel_handler_t*)uart_putc, \
					(void*)(MEM_IMMR | IO_PSC3_OFFSET));

	printf(VT_RESET);
	printf(VT_CLEARSCR);

	printf("tiny_ppc [experimental_001]\n");

	putc('\n');

	printf("core_version : 0x%04x\n", (e300_oea_rd(PVR) >> 16) & 0xffff);
	printf("core_revision: 0x%04x\n", e300_oea_rd(PVR) & 0xffff);
	printf("MBAR: 0x%08x\n\n", e300_oea_rd(MBAR));

	diu_init();
	diu_rd_dpy_mode(&dpy);
	dpy_print_mode(&dpy);
	putc('\n');
	diu_wr_dpy_mode((dpy_mode_t*)&vesa_800x600_60);
	diu_rd_dpy_mode(&dpy);
	dpy_print_mode(&dpy);

	i=800*600;
	ps=(unsigned int*)MEM_GFX_P1_BASE;
	pe=(unsigned int*)genesi_00_800x600_u32_argb;
	while(i--)
		*(ps++)=*(pe++);

	fb_init((unsigned int*)MEM_GFX_P3_BASE, 480, 272);
	/* switch stdout to fbcon */
	channel_register(STDOUT, (channel_handler_t*)chnl_fbputchar, \
					(void*)0x00);

	s29gl_init(MEM_NOR_FLASH_BASE);
	putc('\n');

	/* interrupts */

	ipic = (ipic_t*)(MEM_IMMR | IO_IPIC_OFFSET);

	timer_init((MEM_IMMR | IO_GPT0_OFFSET));
	timer_start((MEM_IMMR | IO_GPT0_OFFSET));
	timer_set_inten((MEM_IMMR | IO_GPT0_OFFSET));

	ipic -> simsr_h = 0x00000002;
	putc('\n');

	/* ... */

	/* never returns */
	pfDoForth(MinDicNames, (const char *)NULL, FALSE);

	/* ... */

	puts("-- waiting for reset --\n");
	getchar();

	/* dirty reset */
	uboot_restart();

}
