/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of this project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <asm/mpc51xx-io.h>
#include <asm/mmap.h>
#include <asm/e300.h>
#include <sysutil.h>
#include <string.h>
#include <diu.h>
#include <vesa.h>

static volatile diu_t* diu;
static volatile clk_ctrl_t* clk;
static int diu_ready = 0;

static void adscpy(diu_ads_t* desc, unsigned int ads_base)
{
	unsigned int *p;
	unsigned int *b;
	unsigned int count;
	
	p = (unsigned int*)ads_base;
	b = (unsigned int*)desc;
	
	count = 9;
	while(count--){
       *(p++) = p_endian_swp32(b++);
	}	
}

/* move this function to dpy.c... */
void dpy_print_mode(dpy_mode_t* dpy)
{
	printf("pixclk: %dkHz\n", dpy -> pxl_clk);	
	printf("hres : %d\tvres : %d\n", dpy -> hres, dpy -> vres);
	printf("hfp  : %d\tvfp  : %d\n", dpy -> hfp, dpy -> vfp);
	printf("hspw : %d\tvspw : %d\n", dpy -> hspw, dpy -> vspw);
	printf("hbp  : %d\tvbp  : %d\n", dpy -> hbp, dpy -> vbp);
	printf("hspol: %d\tvspol: %d\n", dpy -> hspol, dpy -> vspol);
}
/* also add a modeline print alternative */
/* ... */

int diu_rd_dpy_mode(dpy_mode_t* dpy)
{
	if(!diu_ready) return -2;

	/* CLK_CTRL_DIU_DIV_OFFSET = 0 */
	dpy -> pxl_clk = (SYS_CLK_FREQ / 1000) / \
					((clk -> scfr1 & CLK_CTRL_DIU_DIV_MASK) >> 1);

	dpy -> hres = ((diu -> disp_size >> DIU_DELTA_X_OFFSET) \
				& DIU_DELTA_X_MASK);
	/* DIU_FP_H_OFFSET = 0 */
	dpy -> hfp = diu -> hsyn_para & DIU_FP_H_MASK;
	dpy -> hspw = ((diu -> hsyn_para >> DIU_PW_H_OFFSET) \
				& DIU_PW_H_MASK);
	dpy -> hbp = ((diu -> hsyn_para & DIU_BP_H_MASK) \
				& DIU_FP_H_MASK);
	dpy -> hspol = (diu -> syn_pol & DIU_INV_HS) ? 0 : 1;
	dpy -> vres = ((diu -> disp_size >> DIU_DELTA_Y_OFFSET) \
				& DIU_DELTA_Y_MASK);
	/* DIU_FP_V_OFFSET = 0 */
	dpy -> vfp = diu -> vsyn_para & DIU_FP_V_MASK;
	dpy -> vspw = ((diu -> vsyn_para >> DIU_PW_V_OFFSET) \
				& DIU_PW_V_MASK);
	dpy -> vbp = ((diu -> vsyn_para >> DIU_BP_V_OFFSET) \
				& DIU_BP_V_MASK);
	dpy -> vspol = (diu -> syn_pol & DIU_INV_VS) ? 0 : 1;
	return 0;
}

int diu_wr_dpy_mode(dpy_mode_t* dpy)
{
	if(!diu_ready) return -2;

	int val;

	/* clk_inv / dly_num */

	val = clk -> scfr1;
	clk -> scfr1 = (val & ~CLK_CTRL_DIU_DIV_MASK) \
				| (((SYS_CLK_FREQ / 1000) / dpy -> pxl_clk) << 1);

	val = diu -> disp_size;
	diu -> disp_size = (val & ~(DIU_DELTA_X_MASK << DIU_DELTA_X_OFFSET)) \
					| (dpy -> hres << DIU_DELTA_X_OFFSET);
	val = diu -> hsyn_para;
	diu -> hsyn_para = (val & ~(DIU_FP_H_MASK << DIU_FP_H_OFFSET)) \
					| (dpy -> hfp << DIU_FP_H_OFFSET);
	val = diu -> hsyn_para;
	diu -> hsyn_para = (val & ~(DIU_PW_H_MASK << DIU_PW_H_OFFSET)) \
					| (dpy -> hspw << DIU_PW_H_OFFSET);
	val = diu -> hsyn_para;
	diu -> hsyn_para = (val & ~(DIU_BP_H_MASK << DIU_BP_H_OFFSET)) \
					| (dpy -> hbp << DIU_BP_H_OFFSET);
	diu -> syn_pol &= ~DIU_INV_HS;
	diu -> syn_pol |= dpy -> vspol ? 0 : DIU_INV_HS;
	val = diu -> disp_size;

	diu -> disp_size = (val & ~(DIU_DELTA_Y_MASK << DIU_DELTA_Y_OFFSET)) \
					| (dpy -> vres << DIU_DELTA_Y_OFFSET);
	val = diu -> vsyn_para;
	diu -> vsyn_para = (val & ~(DIU_FP_V_MASK << DIU_FP_V_OFFSET)) \
					| (dpy -> vfp << DIU_FP_V_OFFSET);
	val = diu -> vsyn_para;
	diu -> vsyn_para = (val & ~(DIU_PW_V_MASK << DIU_PW_V_OFFSET)) \
					| (dpy -> vspw << DIU_PW_V_OFFSET);
	val = diu -> vsyn_para;
	diu -> vsyn_para = (val & ~(DIU_BP_V_MASK << DIU_BP_V_OFFSET)) \
					| (dpy -> vbp << DIU_BP_V_OFFSET);
	diu -> syn_pol &= ~DIU_INV_VS;
	diu -> syn_pol |= dpy -> vspol ? 0 : DIU_INV_VS;

	return 0;
}

int diu_init()
{
	clk = (clk_ctrl_t*)(MEM_IMMR | IO_CLOCK_OFFSET);

	/* enable DIU clock */
	/* clk -> sccr2 |= CLK_CTRL_DIU_EN; */
	
	diu = (diu_t*)(MEM_IMMR | IO_DIU_OFFSET);

	/* planes initialization */
	/* plane 1 */

	diu_ads_t ad;

	/* argb 8:8:8:8 (no byte flipping) */
	/* ppc native (big endian mode) */

	/* word 0 */
	ad.comp_0 = 0x08;
	ad.comp_1 = 0x08;
	ad.comp_2 = 0x08;
	ad.comp_3 = 0x08;
	ad.pixel_s = 0x03;
	ad.palletized = 0x00;
	ad.red_c = 0x01;
	ad.green_c = 0x02;
	ad.blue_c = 0x03;
	ad.alpha_c = 0x00;
	ad.byte_f = 0x00;
	ad.nmap_0 = 0x00;
	/* word 1 */
	ad.addr = MEM_GFX_P1_BASE;
	/* word 2 */
	ad.deltaxs = 800;	
	ad.nmap_2 = 0x00;
	ad.deltays = 600;
	ad.nmap_1 = 0x00;
	ad.g_alpha = 0x00;	/* alpha ignored on plane 1 */
	/* word 3 */
	ad.deltaxi = 800;
	ad.nmap_4 = 0x00;
	ad.deltayi = 600;
	ad.nmap_3 = 0x00;
	ad.flip = 0x00;
	/* word 4 */
	ad.offset_yi = 0x00;
	ad.offset_xi = 0x00;
	/* word 5 */
	ad.offset_yd = 0x00;
	ad.offset_xd = 0x00;
	/* word 6 */
	/* ck disabled => ckmin/rgb/=255 / ckmax/rgb/=0 */
	ad.ckmax_b = 0x00;
	ad.ckmax_g = 0x00;
	ad.ckmax_r = 0x00;
	/* word 7 */
	ad.ckmin_b = 0xff;
	ad.ckmin_g = 0xff;
	ad.ckmin_r = 0xff;
	/* word 8 */
	ad.next_ad = IO_ADSC_2_BASE;

	memset_32((void*)MEM_GFX_P1_BASE, 0xff000000, 1024*768);
	adscpy(&ad, IO_ADSC_1_BASE);

	diu -> desc_1 = IO_ADSC_1_BASE;

	/* set plane 2 descriptor up */

	/* word 0 */
	/* ... no change */
	/* word 1 */
	ad.addr = MEM_GFX_P2_BASE;
	/* word 2 */
	ad.deltaxs = 480;	
	ad.nmap_2 = 0x00;
	ad.deltays = 272;
	ad.nmap_1 = 0x00;
	ad.g_alpha = 0x00;
	/* word 3 */
	ad.deltaxi = 480;
	ad.nmap_4 = 0x00;
	ad.deltayi = 272;
	ad.nmap_3 = 0x00;
	ad.flip = 0x00;
	/* word 4 */
	ad.offset_yi = 0x00;
	ad.offset_xi = 0x00;
	/* word 5 */
	ad.offset_yd = 275;
	ad.offset_xd = 25;
	/* word 6..7 */
	/* ... no change */
	/* word 8 */
	ad.next_ad = IO_ADSC_3_BASE;

	memset_32((void*)MEM_GFX_P2_BASE, 0x80000000, 1024*768);
	adscpy(&ad, IO_ADSC_2_BASE);

	diu -> desc_2 = IO_ADSC_2_BASE;

	/* set plane 3 descriptor up */

	/* word 0 */
	/* ... no change */
	/* word 1 */
	ad.addr = MEM_GFX_P3_BASE;
	/* word 2 */
	ad.deltaxs = 480;	
	ad.nmap_2 = 0x00;
	ad.deltays = 272;
	ad.nmap_1 = 0x00;
	ad.g_alpha = 0x00;
	/* word 3 */
	ad.deltaxi = 480;
	ad.nmap_4 = 0x00;
	ad.deltayi = 272;
	ad.nmap_3 = 0x00;
	ad.flip = 0x00;
	/* word 4 */
	ad.offset_yi = 0x00;
	ad.offset_xi = 0x00;
	/* word 5 */
	ad.offset_yd = 35;
	ad.offset_xd = 270;
	/* word 6..7 */
	/* ... no change */
	/* word 8 */
	ad.next_ad = 0x00;

	memset_32((void*)MEM_GFX_P3_BASE, 0x80000000, 1024*768);
	adscpy(&ad, IO_ADSC_3_BASE);

	diu -> desc_3 = IO_ADSC_3_BASE;

	/* note: planes can't be switched off by writting 0x00 in DESC_x */
	/* (see errata sheet) */
	
	/* workaround: assign a small AOI but not < 256b */
	/* see datasheet DIU section / page 36 */

	diu_ready = 1;

	return 0;
}
