/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of this project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <asm/mpc51xx-io.h>
#include <asm/mmap.h>
#include <timer.h>

#include <sysutil.h>

void timer_init(unsigned int base /*, unsigned int period*/)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	/* configure prescaler */
	gpt -> ciucor = (60000 & 0xffff) << 16;

	/* configure counter */
	gpt -> ciucor |= 275 & 0xffff;

	/* switch to cpu timer mode */
	gpt -> emsr = 0x00000004;

	printf("%s: gpt/emsr: 0x%08x\n", __FILE__, gpt -> emsr);
	printf("%s: gpt/ciucor: 0x%08x\n", __FILE__, gpt -> ciucor);
}

void timer_start(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	gpt -> emsr |= GPT_CE;

	/* continous mode */
	gpt -> emsr |= GPT_STOP_CONT;

	printf("%s: gpt/emsr: 0x%08x\n", __FILE__, gpt -> emsr);
}

void timer_stop(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	gpt -> emsr &= ~GPT_CE;

	/* continous mode */
	gpt -> emsr &= ~GPT_STOP_CONT;

	printf("0x%08x\n", gpt -> emsr);
}

void timer_clear_texp(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	gpt -> sr |= GPT_TEXP;
}

void timer_set_inten(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	gpt -> emsr |= GPT_INTEN;
}

void timer_clear_inten(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	gpt -> emsr &= ~GPT_INTEN;
}

int timer_sr(unsigned int base)
{
	gpt_t* gpt;
	gpt = (gpt_t*)base;

	/* 	printf("0x%08x\n", gpt -> sr); */

	return(gpt -> sr);
}
