/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#ifndef __MPC51XX_IO_H__
#define __MPC51XX_IO_H__

#include <types.h>

/* clock module */

typedef struct clk_ctrl {
	u32 spmr;			/* 0x00 */
	u32 sccr1;			/* 0x04 */
	u32 sccr2;			/* 0x08 */
	u32 scfr1;			/* 0x0c */
	u32 scfr2;			/* 0x10 */
	u32 reserved_0;		/* 0x14 */
	u32 bcr;			/* 0x18 */
	u32 p0ccr;			/* 0x1c */
	u32 p1ccr;			/* 0x20 */
	u32 p2ccr;			/* 0x24 */
	u32 p3ccr;			/* 0x28 */
	u32 p4ccr;			/* 0x2c */
	u32 p5ccr;			/* 0x30 */
	u32 p6ccr;			/* 0x34 */
	u32 p7ccr;			/* 0x38 */
	u32 p8ccr;			/* 0x3c */
	u32 p9ccr;			/* 0x40 */
	u32 p10ccr;			/* 0x44 */
	u32 p11ccr;			/* 0x48 */
	u32 spccr;			/* 0x4c */
	u32 cccr;			/* 0x50 */
	u32 dccr;			/* 0x54 */
	u32 reserved_1[42];	/* 0x58 ... 0xfc */
} clk_ctrl_t;

/* sccr2 */
#define CLK_CTRL_DIU_EN (1 << 31)
#define CLK_CTRL_AXE_EN (1 << 30)
#define CLK_CTRL_MEM_EN (1 << 29)
#define CLK_CTRL_USB1_EN (1 << 28)
#define CLK_CTRL_USB2_EN (1 << 27)
/* ... */
/* scfr1 */
#define CLK_CTRL_DIU_DIV_MASK	0xff
#define CLK_CTRL_DIU_DIV_OFFSET 0

/* IPIC */

typedef struct ipic {
	u32 sicfr;			/* 0x00 */
	u32 sivcr;			/* 0x04 */
	u32 sipnr_h;		/* 0x08 */
	u32 sipnr_l;		/* 0x0c */
	u32 siprr_a;		/* 0x10 */
	u32 siprr_b;		/* 0x14 */
	u32 siprr_c;		/* 0x18 */
	u32 siprr_d;		/* 0x1c */
	u32 simsr_h;		/* 0x20 */
	u32 simsr_l;		/* 0x24 */
	u32 sicnr;			/* 0x28 */
	u32 sepnr;			/* 0x2c */
	u32 smprr_a;		/* 0x30 */
	u32 smprr_b;		/* 0x34 */
	u32 semsr;			/* 0x38 */
	u32 secnr;			/* 0x3c */
	u32 sersr;			/* 0x40 */
	u32 sermr;			/* 0x44 */
	u32 sercr;			/* 0x48 */
	u32 reserved_0;		/* 0x4c */
	u32 sifcr_h;		/* 0x50 */
	u32 sifcr_l;		/* 0x54 */
	u32 sefcr;			/* 0x58 */
	u32 serfr;			/* 0x5c */
	u32 scvcr;			/* 0x60 */
	u32 smvcr;			/* 0x64 */
	u32 reserved_1[39];	/* 0x68 */
} ipic_t;

/* psc (this structure comes from u-boot) */

typedef struct psc512x {
	volatile u8	mode;		/* PSC + 0x00 */
	volatile u8	res0[3];
	union {				/* PSC + 0x04 */
		volatile u16	status;
		volatile u16	clock_select;
	} sr_csr;
#define psc_status	sr_csr.status
#define psc_clock_select sr_csr.clock_select
	volatile u16	res1;
	volatile u8	command;	/* PSC + 0x08 */
	volatile u8	res2[3];
	union {				/* PSC + 0x0c */
		volatile u8	buffer_8;
		volatile u16	buffer_16;
		volatile u32	buffer_32;
	} buffer;
#define psc_buffer_8	buffer.buffer_8
#define psc_buffer_16	buffer.buffer_16
#define psc_buffer_32	buffer.buffer_32
	union {				/* PSC + 0x10 */
		volatile u8	ipcr;
		volatile u8	acr;
	} ipcr_acr;
#define psc_ipcr	ipcr_acr.ipcr
#define psc_acr		ipcr_acr.acr
	volatile u8	res3[3];
	union {				/* PSC + 0x14 */
		volatile u16	isr;
		volatile u16	imr;
	} isr_imr;
#define psc_isr		isr_imr.isr
#define psc_imr		isr_imr.imr
	volatile u16	res4;
	volatile u8	ctur;		/* PSC + 0x18 */
	volatile u8	res5[3];
	volatile u8	ctlr;		/* PSC + 0x1c */
	volatile u8	res6[3];
	volatile u32	ccr;		/* PSC + 0x20 */
	volatile u8	res7[12];
	volatile u8	ivr;		/* PSC + 0x30 */
	volatile u8	res8[3];
	volatile u8	ip;		/* PSC + 0x34 */
	volatile u8	res9[3];
	volatile u8	op1;		/* PSC + 0x38 */
	volatile u8	res10[3];
	volatile u8	op0;		/* PSC + 0x3c */
	volatile u8	res11[3];
	volatile u32	sicr;		/* PSC + 0x40 */
	volatile u8	res12[60];
	volatile u32	tfcmd;		/* PSC + 0x80 */
	volatile u32	tfalarm;	/* PSC + 0x84 */
	volatile u32	tfstat;		/* PSC + 0x88 */
	volatile u32	tfintstat;	/* PSC + 0x8C */
	volatile u32	tfintmask;	/* PSC + 0x90 */
	volatile u32	tfcount;	/* PSC + 0x94 */
	volatile u16	tfwptr;		/* PSC + 0x98 */
	volatile u16	tfrptr;		/* PSC + 0x9A */
	volatile u32	tfsize;		/* PSC + 0x9C */
	volatile u8	res13[28];
	union {				/* PSC + 0xBC */
		volatile u8	buffer_8;
		volatile u16	buffer_16;
		volatile u32	buffer_32;
	} tfdata_buffer;
#define tfdata_8	tfdata_buffer.buffer_8
#define tfdata_16	tfdata_buffer.buffer_16
#define tfdata_32	tfdata_buffer.buffer_32

	volatile u32	rfcmd;		/* PSC + 0xC0 */
	volatile u32	rfalarm;	/* PSC + 0xC4 */
	volatile u32	rfstat;		/* PSC + 0xC8 */
	volatile u32	rfintstat;	/* PSC + 0xCC */
	volatile u32	rfintmask;	/* PSC + 0xD0 */
	volatile u32	rfcount;	/* PSC + 0xD4 */
	volatile u16	rfwptr;		/* PSC + 0xD8 */
	volatile u16	rfrptr;		/* PSC + 0xDA */
	volatile u32	rfsize;		/* PSC + 0xDC */
	volatile u8	res18[28];
	union {				/* PSC + 0xFC */
		volatile u8	buffer_8;
		volatile u16	buffer_16;
		volatile u32	buffer_32;
	} rfdata_buffer;
#define rfdata_8	rfdata_buffer.buffer_8
#define rfdata_16	rfdata_buffer.buffer_16
#define rfdata_32	rfdata_buffer.buffer_32
} psc512x_t;

/* PSC FIFOC SR */
#define PSC_FIFOC_DATA_READY 0x00000020
#define PSC_FIFOC_EMPTY      0x00000001

/* GPT */

typedef struct gpt {
	u32 emsr;		/* 0x00 */
	u32 ciucor;		/* 0x04 */
	u32 pwmcr;		/* 0x08 */
	u32 sr;			/* 0x0c */
} gpt_t;

#define GPT_INTEN 		0x00000100
#define GPT_OPEN_DRN	0x00000200
#define GPT_STOP_CONT	0x00000400
#define GPT_CE			0x00001000
#define GPT_TEXP		0x00000008

/* DIU */

typedef struct diu {
	u32	desc_1;	    	/* 0x00 */
	u32	desc_2;     	/* 0x04 */
	u32	desc_3;     	/* 0x08 */
	u32	gamma;      	/* 0x0c */
	u32 palette;    	/* 0x10 */
	u32 cursor;     	/* 0x14 */
	u32 cursor_pos;		/* 0x18 */
	u32 diu_mode;		/* 0x1c */
	u32 bgnd;			/* 0x20 */
	u32 bgnd_wb;		/* 0x24 */
	u32	disp_size;		/* 0x28 */
	u32	wb_size;		/* 0x2c */
	u32	wb_mem_addr;	/* 0x30 */
	u32	hsyn_para;		/* 0x34 */
	u32	vsyn_para;		/* 0x38 */
	u32	syn_pol;		/* 0x3c */
	u32	thresholds;		/* 0x40 */
	u32 int_status;		/* 0x44 */
	u32	int_mask;		/* 0x48 */
	u32	colorbar1;		/* 0x4c */
	u32	colorbar2;		/* 0x50 */
	u32	colorbar3;		/* 0x54 */
	u32	colorbar4;		/* 0x58 */
	u32	colorbar5;		/* 0x5c */
	u32	colorbar6;		/* 0x60 */
	u32	colorbar7;		/* 0x64 */
	u32	colorbar8;		/* 0x68 */
	u32 filling;		/* 0x6c */
	u32	plut;			/* 0x70 */
} diu_t;

/* disp_size */
#define DIU_DELTA_X_MASK	0x7ff
#define DIU_DELTA_X_OFFSET	0
#define DIU_DELTA_Y_MASK	0x7ff
#define DIU_DELTA_Y_OFFSET  16
/* hsyn_para */
#define DIU_BP_H_MASK	0x3ff
#define DIU_BP_H_OFFSET 22
#define DIU_PW_H_MASK	0x3ff
#define DIU_PW_H_OFFSET	11
#define DIU_FP_H_MASK	0x3ff
#define DIU_FP_H_OFFSET	0
/* vsyn_para */
#define DIU_BP_V_MASK 0x3ff
#define DIU_BP_V_OFFSET 22
#define DIU_PW_V_MASK 0x3ff
#define DIU_PW_V_OFFSET	11
#define DIU_FP_V_MASK 0x3ff
#define DIU_FP_V_OFFSET	0
/* syn_pol */
#define DIU_BP_VS (1 << 4)
#define DIU_BP_HS (1 << 3)
#define DIU_INV_CS (1 << 2)
#define DIU_INV_VS (1 << 1)
#define DIU_INV_HS (1)

/* DIU / planes Area Descriptor Structure */

typedef volatile struct diu_ads {
	/* word 0*/
	u32 nmap_0 : 3;
	u32 byte_f : 1;
	u32 alpha_c : 3;
	u32 blue_c : 2;
	u32 green_c : 2;
	u32 red_c : 2;
	u32 palletized : 1;
	u32 pixel_s : 2;
	u32 comp_3 : 4;
	u32 comp_2 : 4;
	u32 comp_1 : 4;
	u32 comp_0 : 4;
	/* word 1 */
	u32 addr;
	/* word 2 */
	u32 g_alpha : 8;
	u32 nmap_1 : 1;
	u32 deltays : 11;
	u32 nmap_2 : 1;
	u32 deltaxs : 11;
	/* word 3 */
	u32 flip : 2;
	u32 nmap_3 : 3;
	u32 deltayi : 11;
	u32 nmap_4 : 5;
	u32 deltaxi : 11;
	/* word 4 */
	u32 nmap_5 : 5;
	u32 offset_yi : 11;
	u32 nmap_6 : 5;
	u32 offset_xi : 11;
	/* word 5 */
	u32 nmap_7 : 5;
	u32 offset_yd : 11;
	u32 nmap_8 : 5;
	u32 offset_xd : 11;
	/* word 6 */
	u32 nmap_9 : 8;
	u32 ckmax_b : 8;
	u32 ckmax_g : 8;
	u32 ckmax_r : 8;
	/* word 7 */
	u32 nmap_10 : 8;
	u32 ckmin_b : 8;
	u32 ckmin_g : 8;
	u32 ckmin_r : 8;
	/* word 8 */
	u32 next_ad;
} diu_ads_t;

#endif /* __MPC51XX_IO_H__ */
